@@ -10,7 +10,7 @@ test_svd() {
     # NOTE we care about errors in svd2rust, but not about errors / warnings in rustfmt
     local cwd=$(pwd)
     pushd $td
-    RUST_BACKTRACE=1 $cwd/target/$TARGET/release/svd2rust -i $OPTIONS ${1}.svd
+    RUST_BACKTRACE=1 $cwd/target/$TARGET/release/svd2rust $strict $const_generic -i ${1}.svd
 
     mv lib.rs src/lib.rs
 
@@ -25,7 +25,7 @@ test_svd_for_target() {
     # NOTE we care about errors in svd2rust, but not about errors / warnings in rustfmt
     local cwd=$(pwd)
     pushd $td
-    RUST_BACKTRACE=1 $cwd/target/$TARGET/release/svd2rust --target $1 -i input.svd
+    RUST_BACKTRACE=1 $cwd/target/$TARGET/release/svd2rust $strict $const_generic --target $1 -i input.svd
 
     mv lib.rs src/lib.rs
 
@@ -70,6 +70,25 @@ main() {
             ;;
     esac
 
+    case $OPTIONS in
+        all)
+            const_generic="--const_generic"
+            strict="--strict"
+            ;;
+        strict)
+            const_generic=""
+            strict="--strict"
+            ;;
+        const)
+            const_generic="--const_generic"
+            strict=""
+            ;;
+        *)
+            const_generic=""
+            strict=""
+            ;;
+    esac
+
     # test crate
     cargo init --name foo $td
     echo 'cortex-m = "0.7.0"' >> $td/Cargo.toml
@@ -488,10 +507,10 @@ main() {
         # test other targets (architectures)
         OTHER)
             echo '[dependencies.msp430]' >> $td/Cargo.toml
-            echo 'version = "0.2.2"' >> $td/Cargo.toml
+            echo 'version = "0.3.0"' >> $td/Cargo.toml
 
             # Test MSP430
-            test_svd_for_target msp430 https://raw.githubusercontent.com/pftbest/msp430g2553/v0.1.3-svd/msp430g2553.svd
+            test_svd_for_target msp430 https://raw.githubusercontent.com/pftbest/msp430g2553/v0.3.0-svd/msp430g2553.svd
         ;;
 
         # Community-provided RISC-V SVDs
@@ -512,15 +531,15 @@ main() {
 
         SiliconLabs)
             # #99 regression tests
-            test_svd SIM3C1x4_SVD
-            test_svd SIM3C1x6_SVD
-            test_svd SIM3C1x7_SVD
-            test_svd SIM3L1x4_SVD
-            test_svd SIM3L1x6_SVD
-            test_svd SIM3L1x7_SVD
-            test_svd SIM3U1x4_SVD
-            test_svd SIM3U1x6_SVD
-            test_svd SIM3U1x7_SVD
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3C1x4.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3C1x6.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3C1x7.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3L1x4.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3L1x6.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3L1x7.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3U1x4.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3U1x6.svd
+            test_svd_for_target cortex-m https://raw.githubusercontent.com/posborne/cmsis-svd/master/data/SiliconLabs/SiM3_NRND/SIM3U1x7.svd
 
             # FIXME(???) panicked at "c.text.clone()"
             # test_svd SIM3L1x8_SVD
@@ -589,15 +608,25 @@ main() {
             echo '[dependencies.bare-metal]' >> $td/Cargo.toml
             echo 'version = "1.0.0"' >> $td/Cargo.toml
 
+            echo '[dependencies.riscv]' >> $td/Cargo.toml
+            echo 'version = "0.6.0"' >> $td/Cargo.toml
+
+            echo '[dependencies.riscv-rt]' >> $td/Cargo.toml
+            echo 'version = "0.8.0"' >> $td/Cargo.toml
+
             echo '[dependencies.xtensa-lx]' >> $td/Cargo.toml
-            echo 'version = "0.3.0"' >> $td/Cargo.toml
-            echo 'features = ["lx6"]' >> $td/Cargo.toml
+            echo 'version = "0.6.0"' >> $td/Cargo.toml
+            echo 'features = ["esp32"]' >> $td/Cargo.toml
 
             echo '[dependencies.xtensa-lx-rt]' >> $td/Cargo.toml
-            echo 'version = "0.5.0"' >> $td/Cargo.toml
-            echo 'features = ["lx6"]' >> $td/Cargo.toml
+            echo 'version = "0.9.0"' >> $td/Cargo.toml
+            echo 'features = ["esp32"]' >> $td/Cargo.toml
 
-            test_svd_for_target xtensa-lx https://raw.githubusercontent.com/esp-rs/esp32/master/svd/esp32.svd
+            test_svd_for_target riscv https://raw.githubusercontent.com/espressif/svd/main/svd/esp32c3.svd
+
+            test_svd_for_target xtensa-lx https://raw.githubusercontent.com/espressif/svd/main/svd/esp32.svd
+            test_svd_for_target xtensa-lx https://raw.githubusercontent.com/espressif/svd/main/svd/esp32s2.svd
+            test_svd_for_target xtensa-lx https://raw.githubusercontent.com/espressif/svd/main/svd/esp32s3.svd
         ;;
 
     esac
