@@ -2,16 +2,18 @@ use std::borrow::Cow;
 use std::cmp::Ordering;
 use std::collections::HashMap;
 
-use crate::svd::{Cluster, ClusterInfo, Peripheral, Register, RegisterCluster, RegisterProperties};
-use log::warn;
-use proc_macro2::TokenStream;
-use proc_macro2::{Ident, Punct, Spacing, Span};
+use crate::svd::{
+    Cluster, ClusterInfo, DeriveFrom, DimElement, Peripheral, Register, RegisterCluster,
+    RegisterProperties,
+};
+use log::{debug, trace, warn};
+use proc_macro2::{Ident, Punct, Spacing, Span, TokenStream};
 use quote::{quote, ToTokens};
-use svd_parser::derive_from::DeriveFrom;
 use syn::{parse_str, Token};
 
 use crate::util::{
-    self, Config, FullName, ToSanitizedSnakeCase, ToSanitizedUpperCase, BITS_PER_BYTE,
+    self, handle_cluster_error, handle_reg_error, unsuffixed, Config, FullName,
+    ToSanitizedSnakeCase, ToSanitizedUpperCase, BITS_PER_BYTE,
 };
 use anyhow::{anyhow, bail, Context, Result};
 
@@ -91,6 +93,11 @@ pub fn render(
     // Derived peripherals may not require re-implementation, and will instead
     // use a single definition of the non-derived version.
     if derive_regs {
+        // re-export the base module to allow deriveFrom this one
+        out.extend(quote! {
+            #[doc = #description]
+            pub use #base as #name_sc;
+        });
         return Ok(out);
     }
 
@@ -128,29 +135,12 @@ pub fn render(
 
                 match (erc, ancestor) {
                     (RegisterCluster::Register(reg), RegisterCluster::Register(other_reg)) => {
-                        match other_reg {
-                            Register::Array(ref info, ref array_info) => {
-                                Some(RegisterCluster::Register(Register::Array(
-                                    reg.derive_from(info),
-                                    array_info.clone(),
-                                )))
-                            }
-                            Register::Single(ref info) => Some(RegisterCluster::Register(
-                                Register::Single(reg.derive_from(info)),
-                            )),
-                        }
+                        Some(RegisterCluster::Register(reg.derive_from(other_reg)))
                     }
                     (
                         RegisterCluster::Cluster(cluster),
                         RegisterCluster::Cluster(other_cluster),
-                    ) => match other_cluster {
-                        Cluster::Array(ref info, ref array_info) => Some(RegisterCluster::Cluster(
-                            Cluster::Array(cluster.derive_from(info), array_info.clone()),
-                        )),
-                        Cluster::Single(ref info) => Some(RegisterCluster::Cluster(
-                            Cluster::Single(cluster.derive_from(info)),
-                        )),
-                    },
+                    ) => Some(RegisterCluster::Cluster(cluster.derive_from(other_cluster))),
                     _ => {
                         eprintln!(
                             "{} can't derive from {}",
@@ -178,24 +168,31 @@ pub fn render(
     let defaults = p.default_register_properties.derive_from(defaults);
 
     // Push any register or cluster blocks into the output
+    debug!(
+        "Pushing {} register or cluster blocks into output",
+        ercs.len()
+    );
     let mut mod_items = TokenStream::new();
     mod_items.extend(register_or_cluster_block(&ercs, &defaults, None, config)?);
 
+    debug!("Pushing cluster information into output");
     // Push all cluster related information into the peripheral module
     for c in &clusters {
+        trace!("Cluster: {}", c.name);
         mod_items.extend(cluster_block(c, &defaults, p, all_peripherals, config)?);
     }
 
+    debug!("Pushing register information into output");
     // Push all register related information into the peripheral module
     for reg in registers {
-        mod_items.extend(register::render(
-            reg,
-            registers,
-            p,
-            all_peripherals,
-            &defaults,
-            config,
-        )?);
+        trace!("Register: {}", reg.name);
+        match register::render(reg, registers, p, all_peripherals, &defaults, config) {
+            Ok(rendered_reg) => mod_items.extend(rendered_reg),
+            Err(e) => {
+                let res: Result<TokenStream> = Err(e);
+                return handle_reg_error("Error rendering register", *reg, res);
+            }
+        };
     }
 
     let description =
@@ -222,6 +219,7 @@ struct RegisterBlockField {
     description: String,
     offset: u32,
     size: u32,
+    accessors: Option<TokenStream>,
 }
 
 #[derive(Clone, Debug)]
@@ -438,6 +436,23 @@ impl FieldRegions {
     }
 }
 
+fn make_comment(size: u32, offset: u32, description: &str) -> String {
+    if size > 32 {
+        format!(
+            "0x{:02x}..0x{:02x} - {}",
+            offset,
+            offset + size / 8,
+            util::escape_brackets(&util::respace(description)),
+        )
+    } else {
+        format!(
+            "0x{:02x} - {}",
+            offset,
+            util::escape_brackets(&util::respace(description)),
+        )
+    }
+}
+
 fn register_or_cluster_block(
     ercs: &[RegisterCluster],
     defs: &RegisterProperties,
@@ -446,15 +461,18 @@ fn register_or_cluster_block(
 ) -> Result<TokenStream> {
     let mut rbfs = TokenStream::new();
     let mut accessors = TokenStream::new();
-    let mut have_accessors = false;
 
-    let ercs_expanded = expand(ercs, defs, name, config)?;
+    let ercs_expanded = expand(ercs, defs, name, config)
+        .with_context(|| "Could not expand register or cluster block")?;
 
     // Locate conflicting regions; we'll need to use unions to represent them.
     let mut regions = FieldRegions::default();
 
     for reg_block_field in &ercs_expanded {
         regions.add(reg_block_field)?;
+        if let Some(ts) = &reg_block_field.accessors {
+            accessors.extend(ts.clone());
+        }
     }
 
     // We need to compute the idents of each register/union block first to make sure no conflicts exists.
@@ -478,26 +496,16 @@ fn register_or_cluster_block(
         let is_region_a_union = region.is_union();
 
         for reg_block_field in &region.rbfs {
-            let comment = if reg_block_field.size > 32 {
-                format!(
-                    "0x{:02x}..0x{:02x} - {}",
-                    reg_block_field.offset,
-                    reg_block_field.offset + reg_block_field.size / 8,
-                    util::escape_brackets(util::respace(&reg_block_field.description).as_ref()),
-                )
-            } else {
-                format!(
-                    "0x{:02x} - {}",
-                    reg_block_field.offset,
-                    util::escape_brackets(util::respace(&reg_block_field.description).as_ref()),
-                )
-            };
+            let comment = make_comment(
+                reg_block_field.size,
+                reg_block_field.offset,
+                &reg_block_field.description,
+            );
 
             if is_region_a_union {
                 let name = &reg_block_field.field.ident;
                 let ty = &reg_block_field.field.ty;
                 let offset = reg_block_field.offset as usize;
-                have_accessors = true;
                 accessors.extend(quote! {
                     #[doc = #comment]
                     #[inline(always)]
@@ -558,7 +566,7 @@ fn register_or_cluster_block(
         span,
     );
 
-    let accessors = if have_accessors {
+    let accessors = if !accessors.is_empty() {
         quote! {
             impl #name {
                 #accessors
@@ -589,11 +597,38 @@ fn expand(
 ) -> Result<Vec<RegisterBlockField>> {
     let mut ercs_expanded = vec![];
 
+    debug!("Expanding registers or clusters into Register Block Fields");
     for erc in ercs {
-        ercs_expanded.extend(match &erc {
-            RegisterCluster::Register(register) => expand_register(register, defs, name, config)?,
-            RegisterCluster::Cluster(cluster) => expand_cluster(cluster, defs, name, config)?,
-        });
+        match &erc {
+            RegisterCluster::Register(register) => {
+                match expand_register(register, defs, name, config) {
+                    Ok(expanded_reg) => {
+                        trace!("Register: {}", register.name);
+                        ercs_expanded.extend(expanded_reg);
+                    }
+                    Err(e) => {
+                        let res = Err(e);
+                        return handle_reg_error("Error expanding register", register, res);
+                    }
+                }
+            }
+            RegisterCluster::Cluster(cluster) => {
+                match expand_cluster(cluster, defs, name, config) {
+                    Ok(expanded_cluster) => {
+                        trace!("Cluster: {}", cluster.name);
+                        ercs_expanded.extend(expanded_cluster);
+                    }
+                    Err(e) => {
+                        let res = Err(e);
+                        return handle_cluster_error(
+                            "Error expanding register cluster",
+                            cluster,
+                            res,
+                        );
+                    }
+                }
+            }
+        };
     }
 
     ercs_expanded.sort_by_key(|x| x.offset);
@@ -675,6 +710,7 @@ fn expand_cluster(
             description: info.description.as_ref().unwrap_or(&info.name).into(),
             offset: info.address_offset,
             size: cluster_size,
+            accessors: None,
         }),
         Cluster::Array(info, array_info) => {
             let sequential_addresses = cluster_size == array_info.dim_increment * BITS_PER_BYTE;
@@ -687,15 +723,63 @@ fn expand_cluster(
                     .eq((0..array_info.dim).map(Ok))
             });
 
-            let array_convertible = sequential_indexes && sequential_addresses;
+            let convert_list = match config.keep_list {
+                true => match &array_info.dim_name {
+                    Some(dim_name) => dim_name.contains("[%s]"),
+                    None => info.name.contains("[%s]"),
+                },
+                false => true,
+            };
+
+            let array_convertible = sequential_addresses && convert_list;
 
             if array_convertible {
-                cluster_expanded.push(RegisterBlockField {
-                    field: convert_svd_cluster(&cluster, name)?,
-                    description: info.description.as_ref().unwrap_or(&info.name).into(),
-                    offset: info.address_offset,
-                    size: cluster_size * array_info.dim,
-                });
+                if sequential_indexes {
+                    cluster_expanded.push(RegisterBlockField {
+                        field: convert_svd_cluster(cluster, name)?,
+                        description: info.description.as_ref().unwrap_or(&info.name).into(),
+                        offset: info.address_offset,
+                        size: cluster_size * array_info.dim,
+                        accessors: None,
+                    });
+                } else {
+                    let mut accessors = TokenStream::new();
+                    let nb_name = util::replace_suffix(&info.name, "");
+                    let ty = name_to_ty(&nb_name, name)?;
+                    let nb_name_cs =
+                        Ident::new(&nb_name.to_sanitized_snake_case(), Span::call_site());
+                    let description = info.description.as_ref().unwrap_or(&info.name);
+                    for (i, idx) in array_info.indexes().enumerate() {
+                        let idx_name = Ident::new(
+                            &util::replace_suffix(&info.name, &idx).to_sanitized_snake_case(),
+                            Span::call_site(),
+                        );
+                        let comment = make_comment(
+                            cluster_size,
+                            info.address_offset + (i as u32) * cluster_size / 8,
+                            description,
+                        );
+                        let i = unsuffixed(i as _);
+                        accessors.extend(quote! {
+                            #[doc = #comment]
+                            #[inline(always)]
+                            pub fn #idx_name(&self) -> &#ty {
+                                &self.#nb_name_cs[#i]
+                            }
+                        });
+                    }
+                    cluster_expanded.push(RegisterBlockField {
+                        field: convert_svd_cluster(cluster, name)?,
+                        description: description.into(),
+                        offset: info.address_offset,
+                        size: cluster_size * array_info.dim,
+                        accessors: Some(accessors),
+                    });
+                }
+            } else if sequential_indexes && config.const_generic {
+                // Include a ZST ArrayProxy giving indexed access to the
+                // elements.
+                cluster_expanded.push(array_proxy(info, array_info, name)?);
             } else {
                 for (field_num, field) in expand_svd_cluster(cluster, name)?.iter().enumerate() {
                     cluster_expanded.push(RegisterBlockField {
@@ -703,6 +787,7 @@ fn expand_cluster(
                         description: info.description.as_ref().unwrap_or(&info.name).into(),
                         offset: info.address_offset + field_num as u32 * array_info.dim_increment,
                         size: cluster_size,
+                        accessors: None,
                     });
                 }
             }
@@ -723,16 +808,19 @@ fn expand_register(
     let mut register_expanded = vec![];
 
     let register_size = register
+        .properties
         .size
         .or(defs.size)
         .ok_or_else(|| anyhow!("Register {} has no `size` field", register.name))?;
 
     match register {
         Register::Single(info) => register_expanded.push(RegisterBlockField {
-            field: convert_svd_register(register, name, config.ignore_groups)?,
+            field: convert_svd_register(register, name, config.ignore_groups)
+                .with_context(|| "syn error occured")?,
             description: info.description.clone().unwrap_or_default(),
             offset: info.address_offset,
             size: register_size,
+            accessors: None,
         }),
         Register::Array(info, array_info) => {
             let sequential_addresses = register_size == array_info.dim_increment * BITS_PER_BYTE;
@@ -745,15 +833,60 @@ fn expand_register(
                     .eq((0..array_info.dim).map(Ok))
             });
 
-            let array_convertible = sequential_indexes && sequential_addresses;
+            let convert_list = match config.keep_list {
+                true => match &array_info.dim_name {
+                    Some(dim_name) => dim_name.contains("[%s]"),
+                    None => info.name.contains("[%s]"),
+                },
+                false => true,
+            };
+
+            let array_convertible = sequential_addresses && convert_list;
 
             if array_convertible {
-                register_expanded.push(RegisterBlockField {
-                    field: convert_svd_register(&register, name, config.ignore_groups)?,
-                    description: info.description.clone().unwrap_or_default(),
-                    offset: info.address_offset,
-                    size: register_size * array_info.dim,
-                });
+                if sequential_indexes {
+                    register_expanded.push(RegisterBlockField {
+                        field: convert_svd_register(register, name, config.ignore_groups)?,
+                        description: info.description.clone().unwrap_or_default(),
+                        offset: info.address_offset,
+                        size: register_size * array_info.dim,
+                        accessors: None,
+                    });
+                } else {
+                    let mut accessors = TokenStream::new();
+                    let nb_name = util::replace_suffix(&info.fullname(config.ignore_groups), "");
+                    let ty = name_to_wrapped_ty(&nb_name, name)?;
+                    let nb_name_cs =
+                        Ident::new(&nb_name.to_sanitized_snake_case(), Span::call_site());
+                    let description = info.description.clone().unwrap_or_default();
+                    for (i, idx) in array_info.indexes().enumerate() {
+                        let idx_name = Ident::new(
+                            &util::replace_suffix(&info.fullname(config.ignore_groups), &idx)
+                                .to_sanitized_snake_case(),
+                            Span::call_site(),
+                        );
+                        let comment = make_comment(
+                            register_size,
+                            info.address_offset + (i as u32) * register_size / 8,
+                            &description,
+                        );
+                        let i = unsuffixed(i as _);
+                        accessors.extend(quote! {
+                            #[doc = #comment]
+                            #[inline(always)]
+                            pub fn #idx_name(&self) -> &#ty {
+                                &self.#nb_name_cs[#i]
+                            }
+                        });
+                    }
+                    register_expanded.push(RegisterBlockField {
+                        field: convert_svd_register(register, name, config.ignore_groups)?,
+                        description,
+                        offset: info.address_offset,
+                        size: register_size * array_info.dim,
+                        accessors: Some(accessors),
+                    });
+                }
             } else {
                 for (field_num, field) in expand_svd_register(register, name, config.ignore_groups)?
                     .iter()
@@ -764,6 +897,7 @@ fn expand_register(
                         description: info.description.clone().unwrap_or_default(),
                         offset: info.address_offset + field_num as u32 * array_info.dim_increment,
                         size: register_size,
+                        accessors: None,
                     });
                 }
             }
@@ -804,14 +938,17 @@ fn cluster_block(
     // Generate definition for each of the registers.
     let registers = util::only_registers(&c.children);
     for reg in &registers {
-        mod_items.extend(register::render(
-            reg,
-            &registers,
-            p,
-            all_peripherals,
-            &defaults,
-            config,
-        )?);
+        match register::render(reg, &registers, p, all_peripherals, &defaults, config) {
+            Ok(rendered_reg) => mod_items.extend(rendered_reg),
+            Err(e) => {
+                let res: Result<TokenStream> = Err(e);
+                return handle_reg_error(
+                    "Error generating register definition for a register cluster",
+                    *reg,
+                    res,
+                );
+            }
+        };
     }
 
     // Generate the sub-cluster blocks.
@@ -837,28 +974,16 @@ fn expand_svd_register(
     register: &Register,
     name: Option<&str>,
     ignore_group: bool,
-) -> Result<Vec<syn::Field>, syn::Error> {
+) -> Result<Vec<syn::Field>> {
     let mut out = vec![];
 
     match register {
         Register::Single(_info) => out.push(convert_svd_register(register, name, ignore_group)?),
         Register::Array(info, array_info) => {
-            let indices = array_info
-                .dim_index
-                .as_ref()
-                .map(|v| Cow::from(&**v))
-                .unwrap_or_else(|| {
-                    Cow::from(
-                        (0..array_info.dim)
-                            .map(|i| i.to_string())
-                            .collect::<Vec<_>>(),
-                    )
-                });
-
             let ty_name = util::replace_suffix(&info.fullname(ignore_group), "");
 
-            for (idx, _i) in indices.iter().zip(0..) {
-                let nb_name = util::replace_suffix(&info.fullname(ignore_group), idx);
+            for idx in array_info.indexes() {
+                let nb_name = util::replace_suffix(&info.fullname(ignore_group), &idx);
 
                 let ty = name_to_wrapped_ty(&ty_name, name)?;
 
@@ -874,13 +999,14 @@ fn convert_svd_register(
     register: &Register,
     name: Option<&str>,
     ignore_group: bool,
-) -> Result<syn::Field, syn::Error> {
+) -> Result<syn::Field> {
     Ok(match register {
         Register::Single(info) => {
             let info_name = info.fullname(ignore_group);
             new_syn_field(
                 &info_name.to_sanitized_snake_case(),
-                name_to_wrapped_ty(&info_name, name)?,
+                name_to_wrapped_ty(&info_name, name)
+                    .with_context(|| format!("Error converting info name {}", info_name))?,
             )
         }
         Register::Array(info, array_info) => {
@@ -896,6 +1022,31 @@ fn convert_svd_register(
     })
 }
 
+/// Return an syn::Type for an ArrayProxy.
+fn array_proxy(
+    info: &ClusterInfo,
+    array_info: &DimElement,
+    name: Option<&str>,
+) -> Result<RegisterBlockField, syn::Error> {
+    let ty_name = util::replace_suffix(&info.name, "");
+    let tys = name_to_ty_str(&ty_name, name);
+
+    let ap_path = parse_str::<syn::TypePath>(&format!(
+        "crate::ArrayProxy<{}, {}, {}>",
+        tys,
+        array_info.dim,
+        util::hex(array_info.dim_increment as u64)
+    ))?;
+
+    Ok(RegisterBlockField {
+        field: new_syn_field(&ty_name.to_sanitized_snake_case(), ap_path.into()),
+        description: info.description.as_ref().unwrap_or(&info.name).into(),
+        offset: info.address_offset,
+        size: 0,
+        accessors: None,
+    })
+}
+
 /// Takes a svd::Cluster which may contain a register array, and turn in into
 /// a list of syn::Field where the register arrays have been expanded.
 fn expand_svd_cluster(
@@ -907,22 +1058,10 @@ fn expand_svd_cluster(
     match &cluster {
         Cluster::Single(_info) => out.push(convert_svd_cluster(cluster, name)?),
         Cluster::Array(info, array_info) => {
-            let indices = array_info
-                .dim_index
-                .as_ref()
-                .map(|v| Cow::from(&**v))
-                .unwrap_or_else(|| {
-                    Cow::from(
-                        (0..array_info.dim)
-                            .map(|i| i.to_string())
-                            .collect::<Vec<_>>(),
-                    )
-                });
-
             let ty_name = util::replace_suffix(&info.name, "");
 
-            for (idx, _i) in indices.iter().zip(0..) {
-                let nb_name = util::replace_suffix(&info.name, idx);
+            for idx in array_info.indexes() {
+                let nb_name = util::replace_suffix(&info.name, &idx);
 
                 let ty = name_to_ty(&ty_name, name)?;
 
@@ -982,7 +1121,7 @@ fn name_to_ty_str<'a, 'b>(name: &'a str, ns: Option<&'b str>) -> Cow<'a, str> {
 }
 
 fn name_to_ty(name: &str, ns: Option<&str>) -> Result<syn::Type, syn::Error> {
-    let ident = name_to_ty_str(&name, ns);
+    let ident = name_to_ty_str(name, ns);
     Ok(syn::Type::Path(parse_str::<syn::TypePath>(&ident)?))
 }
 
@@ -1003,7 +1142,16 @@ fn name_to_wrapped_ty_str(name: &str, ns: Option<&str>) -> String {
     }
 }
 
-fn name_to_wrapped_ty(name: &str, ns: Option<&str>) -> Result<syn::Type, syn::Error> {
+fn name_to_wrapped_ty(name: &str, ns: Option<&str>) -> Result<syn::Type> {
     let ident = name_to_wrapped_ty_str(name, ns);
-    Ok(syn::Type::Path(parse_str::<syn::TypePath>(&ident)?))
+    match parse_str::<syn::TypePath>(&ident) {
+        Ok(path) => Ok(syn::Type::Path(path)),
+        Err(e) => {
+            let mut res = Err(e.into());
+            res = res.with_context(|| {
+                format!("Determining syn::TypePath from ident \"{}\" failed", ident)
+            });
+            res
+        }
+    }
 }
